# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import re
import shutil
import six

from itertools import product
from subprocess import PIPE

from law.util import interruptable_popen, colored

from .MergeFastProd import MergeFastProd

from analysis.framework import Task

logger = logging.getLogger(__name__)


class MergeFinal(Task, law.LocalWorkflow):

    merge_dir = luigi.Parameter(default='grids')

    final_tables = luigi.DictParameter()
    observables = luigi.ListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        return {
            i : {'order': order, 'channels': channels, 'obs': obs}
            for i, ((order, channels), obs) in enumerate(product(
                six.iteritems(self.final_tables),
                self.observables
            ))
        }

    def workflow_requires(self):
        return {'mergefastprod': MergeFastProd()}

    def requires(self):
        return MergeFastProd()

    def output(self):
        return law.LocalFileTarget(
            '{}/{}/Combined/Final/{}.{}.{}.tab.gz'.format(
                self.merge_dir, self.name, self.process, self.branch_data['order'],
                self.branch_data['obs']
            )
        )

    def run(self):
        # ensure output parent directory exists
        logger.debug("Creating output parent directory: %s", self.output().parent.path)
        self.output().parent.touch()

        outfile = self.output().path
        logfile = re.sub('.tab.gz$', '.merge2.log', outfile)
        errfile = re.sub('.tab.gz$', '.merge2.err', outfile)

        tablestring = ' '.join(
            '{}/{}/Combined/Final/{}.{}.{}.tab.gz '.format(
                self.merge_dir, self.name, self.process, channel, self.branch_data['obs'])
            for channel in self.branch_data['channels']
        )

        command_line = 'fnlo-tk-merge2 -add {} {}'.format(tablestring, outfile)

        logger.info("Running command: %s", colored(command_line, 'yellow'))
        code, out, error = interruptable_popen(command_line, shell=True, stdout=PIPE, stderr=PIPE)

        logger.debug("Return code: %d", code)
        logger.debug("Standard output:\n%s\n", out)
        logger.debug("Standard error:\n%s\n", error)

        logger.debug("Dumping standard out to file:  %s", logfile)
        with open(logfile, 'w') as f:
            f.write(out)

        if error:
            logger.debug("Dumping standard error to file:  %s", errfile)
            with open(errfile, 'w') as f:
                f.write(error)

        if (code == 0):
            logger.info("fnlo-tk-merge2 status: SUCCESS")
        else:
            error_message = "fnlo-tk-merge2 status: FAIL (exit code {})".format(code)
            logger.error(error_message)
            raise RuntimeError(error_message)
