# -*- coding: utf-8 -*-
import glob
import law
import luigi
import logging
import os
import shutil

from itertools import product
from subprocess import PIPE

from law.util import interruptable_popen, colored

from .FastProd import FastProd
from .FnloCppread import FnloCppread

from analysis.framework import Task, SpaceSeparatedListParameter

logger = logging.getLogger(__name__)


class SingleGridClosure(Task, law.LocalWorkflow):
    command = 'fastnnlo_scaleclosure.py'

    force = luigi.BoolParameter(default=False)

    merge_dir = luigi.Parameter(default='grids')
    plots_dir = luigi.Parameter(default='plots')

    channels = SpaceSeparatedListParameter(
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RRa", "RRb", "RV", "VV")
    )
    observables = luigi.ListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output directory."""
        return {
            i : {'channel': ch, 'index': ch_index, 'observable': obs, 'seed': FastProd().starting_seeds[ch_index]}
            for i, ((ch_index, ch), obs) in enumerate(product(
                enumerate(self.channels), self.observables,
            ))
        }

    def workflow_requires(self):
        if self.force:
            return {"fnlocppread": FnloCppread.req(self, acceptance=0)}
        else:
            return {"fnlocppread": FnloCppread.req(self)}

    def requires(self):
        return {
            'fnlocppread': FnloCppread(branch=self.branch_data['index']),
        }

    def output(self):
        return law.LocalDirectoryTarget(
            '{}/{}/{}/{}/{}'.format(
                self.plots_dir, self.name, self.__class__.__name__,
                self.branch_data['channel'],
                self.branch_data['observable']
            )
        )

    def run(self):
        tabglob = (
            "{self.merge_dir}/{self.name}/{self.branch_data[channel]}/"
            "*.{self.branch_data[observable]}.s{self.branch_data[seed]}.tab.gz"
        ).format(self=self)
        logger.debug("Table glob: %s", tabglob)
        tables = glob.glob(tabglob)

        if not tables:
            raise RuntimeError('No matching file for table glob {}.'.format(tabglob))

        # Only take the first one from all matches
        tabfile = tables[0]
        logger.debug("Table file: %s", tabfile)

        try:
            with self.output().temporary_path() as self.temp_output_path:
                os.mkdir(self.temp_output_path)

                outfile = '{}/{}.{}.{}'.format(
                    self.temp_output_path, self.process,
                    self.branch_data['channel'],
                    self.branch_data['observable']
                )

                command_line_args = [
                    self.command, tabfile,
                    '-f', outfile
                ]

                logger.info("Running command: %s",
                    colored(' '.join(command_line_args), 'yellow'))
                code, out, error = interruptable_popen(command_line_args,
                                                       stdout=PIPE,
                                                       stderr=PIPE)

                logger.debug("Return code: %d", code)
                logger.debug("Standard output:\n%s\n", out)
                logger.debug("Standard error:\n%s\n", error)

                if (code == 0):
                    logger.info("%s status: SUCCESS", self.command)
                else:
                    raise RuntimeError("{} status: FAIL (exit code {})".format(self.command, code))

        # mark job as failed on error, make log entry and reraise
        except Exception as e:
            shutil.rmtree(self.temp_output_path)
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
